@@ -29,15 +29,8 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "largeint95.h"
 #endif /* !DJGPP */
 
-#include "afsdicon.h"
-
 #include "cm.h"
 
-#include "krb.h"
-#include "krb_prot.h"
-/*#include <crypt.h>*/
-#include <afs/prs_fs.h>
-
 #include <osi.h>
 #include "cm_config.h"
 #include "cm_user.h"
@@ -66,16 +59,21 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "afsd_init.h"
 #ifdef DJGPP
 #include "afs/afsmsg95.h"
+#else
+#include "afsd_eventlog.h"
 #endif
 
 #include <afs/vldbint.h>
 #include <afs/afsint.h>
+#include <afs/prs_fs.h>
 
-#define AFS_DAEMON_SERVICE_NAME "TransarcAFSDaemon"
-#define AFS_DAEMON_EVENT_NAME "AFS Client"
+#define AFS_DAEMON_SERVICE_NAME AFSREG_CLT_SVC_NAME
+#define AFS_DAEMON_EVENT_NAME   AFSREG_CLT_SW_NAME
 
 void afs_exit();
 
+extern void afsi_log(char *pattern, ...);
+
 /* globals from the base afsd */
 
 extern int cm_logChunkSize;
@@ -125,12 +123,12 @@ extern long rx_mtu;
 
 extern HANDLE WaitToTerminate;
 
-#define CAPABILITY_ERRORTRANS (1<<0)
-#define CAPABILITY_BITS 1
-
 #undef  DFS_SUPPORT
 #define LOG_PACKET 1
 #undef  NOTSERVICE
+#define LOCK_TESTING 1
+
+#define WORKER_THREADS 10
 
 #define AFSD_HOOK_DLL  "afsdhook.dll"
 #define AFSD_INIT_HOOK "AfsdInitHook"
