@@ -13,7 +13,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_user.c,v 1.14 2003/07/15 23:16:10 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_user.c,v 1.24.4.1 2006/12/17 01:23:30 jaltman Exp $");
 
 # include <sys/types.h>
 # include <errno.h>
@@ -66,8 +66,8 @@ RCSID
  */
 
 pthread_mutex_t rx_if_init_mutex;
-#define LOCK_IF_INIT assert(pthread_mutex_lock(&rx_if_init_mutex)==0);
-#define UNLOCK_IF_INIT assert(pthread_mutex_unlock(&rx_if_init_mutex)==0);
+#define LOCK_IF_INIT assert(pthread_mutex_lock(&rx_if_init_mutex)==0)
+#define UNLOCK_IF_INIT assert(pthread_mutex_unlock(&rx_if_init_mutex)==0)
 
 /*
  * The rx_if_mutex mutex protects the following global variables:
@@ -77,8 +77,8 @@ pthread_mutex_t rx_if_init_mutex;
  */
 
 pthread_mutex_t rx_if_mutex;
-#define LOCK_IF assert(pthread_mutex_lock(&rx_if_mutex)==0);
-#define UNLOCK_IF assert(pthread_mutex_unlock(&rx_if_mutex)==0);
+#define LOCK_IF assert(pthread_mutex_lock(&rx_if_mutex)==0)
+#define UNLOCK_IF assert(pthread_mutex_unlock(&rx_if_mutex)==0)
 #else
 #define LOCK_IF_INIT
 #define UNLOCK_IF_INIT
@@ -94,13 +94,15 @@ pthread_mutex_t rx_if_mutex;
  * failure. Port must be in network byte order.	
  */
 osi_socket
-rxi_GetUDPSocket(u_short port)
+rxi_GetHostUDPSocket(u_int ahost, u_short port)
 {
     int binds, code = 0;
     osi_socket socketFd = OSI_NULLSOCKET;
     struct sockaddr_in taddr;
     char *name = "rxi_GetUDPSocket: ";
-    int greedy = 0;
+#ifdef AFS_LINUX22_ENV
+    int pmtu=IP_PMTUDISC_DONT;
+#endif
 
 #if !defined(AFS_NT40_ENV) && !defined(AFS_DJGPP_ENV)
     if (ntohs(port) >= IPPORT_RESERVED && ntohs(port) < IPPORT_USERRESERVED) {
@@ -121,7 +123,7 @@ rxi_GetUDPSocket(u_short port)
 	goto error;
     }
 
-    taddr.sin_addr.s_addr = 0;
+    taddr.sin_addr.s_addr = ahost;
     taddr.sin_family = AF_INET;
     taddr.sin_port = (u_short) port;
 #ifdef STRUCT_SOCKADDR_HAS_SA_LEN
@@ -147,15 +149,16 @@ rxi_GetUDPSocket(u_short port)
     fcntl(socketFd, F_SETFD, 1);
 #endif
 
+#ifndef AFS_DJGPP_ENV
     /* Use one of three different ways of getting a socket buffer expanded to
      * a reasonable size.
      */
     {
+	int greedy = 0;
 	int len1, len2;
 
 	len1 = 32766;
 	len2 = rx_UdpBufSize;
-#ifndef AFS_DJGPP_ENV
 	greedy =
 	    (setsockopt
 	     (socketFd, SOL_SOCKET, SO_RCVBUF, (char *)&len2,
@@ -172,14 +175,19 @@ rxi_GetUDPSocket(u_short port)
 	    (setsockopt
 	     (socketFd, SOL_SOCKET, SO_RCVBUF, (char *)&len2,
 	      sizeof(len2)) >= 0);
-#endif /* AFS_DJGPP_ENV */
+	if (!greedy)
+	    (osi_Msg "%s*WARNING* Unable to increase buffering on socket\n",
+	     name);
+	MUTEX_ENTER(&rx_stats_mutex);
+	rx_stats.socketGreedy = greedy;
+	MUTEX_EXIT(&rx_stats_mutex);
     }
-
-#ifndef AFS_DJGPP_ENV
-    if (!greedy)
-	(osi_Msg "%s*WARNING* Unable to increase buffering on socket\n",
-	 name);
 #endif /* AFS_DJGPP_ENV */
+
+#ifdef AFS_LINUX22_ENV
+    setsockopt(socketFd, SOL_IP, IP_MTU_DISCOVER, &pmtu, sizeof(pmtu));
+#endif
+
     if (rxi_Listen(socketFd) < 0) {
 	goto error;
     }
@@ -195,14 +203,18 @@ rxi_GetUDPSocket(u_short port)
 	close(socketFd);
 #endif
 
-    MUTEX_ENTER(&rx_stats_mutex);
-    rx_stats.socketGreedy = greedy;
-    MUTEX_EXIT(&rx_stats_mutex);
     return OSI_NULLSOCKET;
 }
 
+osi_socket
+rxi_GetUDPSocket(u_short port)
+{
+    return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
+}
+
 void
-osi_Panic(char *msg, int a1, int a2, int a3)
+osi_Panic(msg, a1, a2, a3) 
+     char *msg; 
 {
     (osi_Msg "Fatal Rx error: ");
     (osi_Msg msg, a1, a2, a3);
@@ -218,50 +230,57 @@ osi_Panic(char *msg, int a1, int a2, int a3)
 void
 osi_AssertFailU(const char *expr, const char *file, int line)
 {
-    osi_Panic("assertion failed: %s, file: %s, line: %d\n", (int)expr,
-	      (int)file, line);
+    osi_Panic("assertion failed: %s, file: %s, line: %d\n", expr,
+	      file, line);
 }
 
-#ifdef	AFS_AIX32_ENV
+#if defined(AFS_AIX32_ENV) && !defined(KERNEL)
 #ifndef osi_Alloc
 static const char memZero;
-char *
+void *
 osi_Alloc(afs_int32 x)
 {
     /* 
-     * 0-length allocs may return NULL ptr from osi_kalloc, so we special-case
+     * 0-length allocs may return NULL ptr from malloc, so we special-case
      * things so that NULL returned iff an error occurred 
      */
     if (x == 0)
-	return &memZero;
-    return ((char *)malloc(x));
+	return (void *)&memZero;
+    return(malloc(x));
 }
 
 void
-osi_Free(char *x, afs_int32 size)
+osi_Free(void *x, afs_int32 size)
 {
     if (x == &memZero)
 	return;
-    free((char *)x);
+    free(x);
 }
 #endif
-#endif /* AFS_AIX32_ENV */
+#endif /* defined(AFS_AIX32_ENV) && !defined(KERNEL) */
 
 #define	ADDRSPERSITE	16
 
 
-afs_uint32 rxi_NetAddrs[ADDRSPERSITE];	/* host order */
+static afs_uint32 rxi_NetAddrs[ADDRSPERSITE];	/* host order */
 static int myNetMTUs[ADDRSPERSITE];
 static int myNetMasks[ADDRSPERSITE];
 static int myNetFlags[ADDRSPERSITE];
-u_int rxi_numNetAddrs;
+static u_int rxi_numNetAddrs;
 static int Inited = 0;
 
 #if defined(AFS_NT40_ENV) || defined(AFS_DJGPP_ENV)
 int
 rxi_getaddr(void)
 {
-    if (rxi_numNetAddrs > 0)
+    /* The IP address list can change so we must query for it */
+    rx_GetIFInfo();
+
+    /* we don't want to use the loopback adapter which is first */
+    /* this is a bad bad hack */
+    if (rxi_numNetAddrs > 1)
+	return htonl(rxi_NetAddrs[1]);  
+    else if (rxi_numNetAddrs > 0)
 	return htonl(rxi_NetAddrs[0]);
     else
 	return 0;
@@ -275,56 +294,66 @@ rxi_getaddr(void)
 int
 rx_getAllAddr(afs_int32 * buffer, int maxSize)
 {
-    int count = 0;
-    for (count = 0; count < rxi_numNetAddrs && maxSize > 0;
-	 count++, maxSize--)
-	buffer[count] = htonl(rxi_NetAddrs[count]);
+    int count = 0, offset = 0;
+
+    /* The IP address list can change so we must query for it */
+    rx_GetIFInfo();
+
+#ifndef AFS_NT40_ENV
+    /* we don't want to use the loopback adapter which is first */
+    /* this is a bad bad hack.
+     * and doesn't hold true on Windows.
+     */
+    if ( rxi_numNetAddrs > 1 )
+        offset = 1;
+#endif /* AFS_NT40_ENV */
+
+    for (count = 0; offset < rxi_numNetAddrs && maxSize > 0;
+	 count++, offset++, maxSize--)
+	buffer[count] = htonl(rxi_NetAddrs[offset]);
 
     return count;
 }
-
 #endif
 
 #ifdef AFS_NT40_ENV
-
 void
 rx_GetIFInfo(void)
 {
-    LOCK_IF_INIT if (Inited) {
-	UNLOCK_IF_INIT return;
-    } else {
-	u_int maxsize;
-	u_int rxsize;
-	int npackets, ncbufs;
-	afs_uint32 i;
+    u_int maxsize;
+    u_int rxsize;
+    int npackets, ncbufs;
+    afs_uint32 i;
 
-	Inited = 1;
-	UNLOCK_IF_INIT rxi_numNetAddrs = ADDRSPERSITE;
+    LOCK_IF_INIT;
+    Inited = 1;
+    UNLOCK_IF_INIT;
 
-	LOCK_IF(void) syscfg_GetIFInfo(&rxi_numNetAddrs, rxi_NetAddrs,
-				       myNetMasks, myNetMTUs, myNetFlags);
+    LOCK_IF;
+    rxi_numNetAddrs = ADDRSPERSITE;
+    (void)syscfg_GetIFInfo(&rxi_numNetAddrs, rxi_NetAddrs,
+                           myNetMasks, myNetMTUs, myNetFlags);
 
-	for (i = 0; i < rxi_numNetAddrs; i++) {
-	    rxsize = rxi_AdjustIfMTU(myNetMTUs[i] - RX_IPUDP_SIZE);
-	    maxsize =
-		rxi_nRecvFrags * rxsize + (rxi_nRecvFrags - 1) * UDP_HDR_SIZE;
-	    maxsize = rxi_AdjustMaxMTU(rxsize, maxsize);
-	    if (rx_maxReceiveSize < maxsize) {
-		rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxsize);
-		rx_maxReceiveSize =
-		    MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
-	    }
+    for (i = 0; i < rxi_numNetAddrs; i++) {
+        rxsize = rxi_AdjustIfMTU(myNetMTUs[i] - RX_IPUDP_SIZE);
+        maxsize =
+            rxi_nRecvFrags * rxsize + (rxi_nRecvFrags - 1) * UDP_HDR_SIZE;
+        maxsize = rxi_AdjustMaxMTU(rxsize, maxsize);
+        if (rx_maxReceiveSize < maxsize) {
+            rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxsize);
+            rx_maxReceiveSize =
+                MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
+        }
 
-	}
-	UNLOCK_IF ncbufs = (rx_maxJumboRecvSize - RX_FIRSTBUFFERSIZE);
-	if (ncbufs > 0) {
-	    ncbufs = ncbufs / RX_CBUFFERSIZE;
-	    npackets = rx_initSendWindow - 1;
-	    rxi_MorePackets(npackets * (ncbufs + 1));
-	}
+    }
+    UNLOCK_IF;
+    ncbufs = (rx_maxJumboRecvSize - RX_FIRSTBUFFERSIZE);
+    if (ncbufs > 0) {
+        ncbufs = ncbufs / RX_CBUFFERSIZE;
+        npackets = rx_initSendWindow - 1;
+        rxi_MorePackets(npackets * (ncbufs + 1));
     }
 }
-
 #endif
 
 static afs_uint32
@@ -385,16 +414,21 @@ rx_GetIFInfo(void)
     struct sockaddr_in *a;
 #endif /* AFS_DJGPP_ENV */
 
-    LOCK_IF_INIT if (Inited) {
-	UNLOCK_IF_INIT return;
+    LOCK_IF_INIT;
+    if (Inited) {
+	UNLOCK_IF_INIT;
+	return;
     }
     Inited = 1;
-    UNLOCK_IF_INIT LOCK_IF rxi_numNetAddrs = 0;
+    UNLOCK_IF_INIT;
+    LOCK_IF;
+    rxi_numNetAddrs = 0;
     memset(rxi_NetAddrs, 0, sizeof(rxi_NetAddrs));
     memset(myNetFlags, 0, sizeof(myNetFlags));
     memset(myNetMTUs, 0, sizeof(myNetMTUs));
     memset(myNetMasks, 0, sizeof(myNetMasks));
-    UNLOCK_IF s = socket(AF_INET, SOCK_DGRAM, 0);
+    UNLOCK_IF;
+    s = socket(AF_INET, SOCK_DGRAM, 0);
     if (s < 0)
 	return;
 
@@ -415,10 +449,10 @@ rx_GetIFInfo(void)
 	return;
     }
 
-    LOCK_IF
+    LOCK_IF;
 #ifdef	AFS_AIX41_ENV
 #define size(p) MAX((p).sa_len, sizeof(p))
-	cplim = buf + ifc.ifc_len;	/*skip over if's with big ifr_addr's */
+    cplim = buf + ifc.ifc_len;	/*skip over if's with big ifr_addr's */
     for (cp = buf; cp < cplim;
 	 cp += sizeof(ifr->ifr_name) + MAX(a->sin_len, sizeof(*a))) {
 	if (rxi_numNetAddrs >= ADDRSPERSITE)
@@ -426,7 +460,7 @@ rx_GetIFInfo(void)
 
 	ifr = (struct ifreq *)cp;
 #else
-	len = ifc.ifc_len / sizeof(struct ifreq);
+    len = ifc.ifc_len / sizeof(struct ifreq);
     if (len > ADDRSPERSITE)
 	len = ADDRSPERSITE;
 
@@ -546,7 +580,8 @@ rx_GetIFInfo(void)
 	    ++rxi_numNetAddrs;
 	}
     }
-    UNLOCK_IF close(s);
+    UNLOCK_IF;
+    close(s);
 
     /* have to allocate at least enough to allow a single packet to reach its
      * maximum size, so ReadPacket will work.  Allocate enough for a couple
@@ -588,15 +623,17 @@ rxi_InitPeerParams(struct rx_peer *pp)
 
 
 
-    LOCK_IF_INIT if (!Inited) {
-	UNLOCK_IF_INIT
-	    /*
-	     * there's a race here since more than one thread could call
-	     * rx_GetIFInfo.  The race stops in rx_GetIFInfo.
-	     */
-	    rx_GetIFInfo();
+    LOCK_IF_INIT;
+    if (!Inited) {
+	UNLOCK_IF_INIT;
+	/*
+	 * there's a race here since more than one thread could call
+	 * rx_GetIFInfo.  The race stops in rx_GetIFInfo.
+	 */
+	rx_GetIFInfo();
     } else {
-    UNLOCK_IF_INIT}
+	UNLOCK_IF_INIT;
+    }
 
 #ifdef ADAPT_MTU
     /* try to second-guess IP, and identify which link is most likely to
@@ -610,7 +647,8 @@ rxi_InitPeerParams(struct rx_peer *pp)
      * pp->burstSize pp->burst pp->burstWait.sec pp->burstWait.usec
      * pp->timeout.usec */
 
-    LOCK_IF for (ix = 0; ix < rxi_numNetAddrs; ++ix) {
+    LOCK_IF;
+    for (ix = 0; ix < rxi_numNetAddrs; ++ix) {
 	if ((rxi_NetAddrs[ix] & myNetMasks[ix]) == (ppaddr & myNetMasks[ix])) {
 #ifdef IFF_POINTOPOINT
 	    if (myNetFlags[ix] & IFF_POINTOPOINT)
@@ -623,14 +661,15 @@ rxi_InitPeerParams(struct rx_peer *pp)
 		pp->ifMTU = MIN(rx_MyMaxSendSize, rxmtu);
 	}
     }
-    UNLOCK_IF if (!pp->ifMTU) {	/* not local */
+    UNLOCK_IF;
+    if (!pp->ifMTU) {		/* not local */
 	pp->timeout.sec = 3;
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(rx_MyMaxSendSize, RX_REMOTE_PACKET_SIZE);
     }
 #else /* ADAPT_MTU */
     pp->rateFlag = 2;		/* start timing after two full packets */
     pp->timeout.sec = 2;
-    pp->ifMTU = OLD_MAX_PACKET_SIZE;
+    pp->ifMTU = MIN(rx_MyMaxSendSize, OLD_MAX_PACKET_SIZE);
 #endif /* ADAPT_MTU */
     pp->ifMTU = rxi_AdjustIfMTU(pp->ifMTU);
     pp->maxMTU = OLD_MAX_PACKET_SIZE;	/* for compatibility with old guys */
@@ -656,3 +695,11 @@ rx_SetNoJumbo(void)
     rx_maxReceiveSize = OLD_MAX_PACKET_SIZE;
     rxi_nSendFrags = rxi_nRecvFrags = 1;
 }
+
+/* Override max MTU.  If rx_SetNoJumbo is called, it must be 
+   called before calling rx_SetMaxMTU since SetNoJumbo clobbers rx_maxReceiveSize */
+void
+rx_SetMaxMTU(int mtu)
+{
+    rx_MyMaxSendSize = rx_maxReceiveSizeUser = rx_maxReceiveSize = mtu;
+}
